import { nextTick, onDeactivated, onMounted, ref } from 'vue';
import { SignJWT } from 'jose';

import './libs/duix.js';
import { questionStreamByPost } from '/@/api/ai/chat';
import { activeGroupType, activeRoomId } from '/@/stores/chatRoom';
import { markdownToTxt } from 'markdown-to-txt';

export type UseDigitalHumanProps = {
	container: string;
};

export const useDigitalHuman = (props: UseDigitalHumanProps) => {
	const { container } = props;
	const duixConfig = {
		appId: '1356792813207031808',
		appKey: '659b068e-900c-4fe5-bb96-3ca70fe0aae4',
		sign: '',
		conversationId: '1909088110274277378',
		/** @description 过期时间（小时） */
		expired: 12,
	};

	// 是否已接口相应
	const isReceiveRes = ref(false);
	const humanIsLoading = ref(true);

	const digitalHumanIsShow = ref(false);
	const closeDigitalHuman = () => {
		digitalHumanIsShow.value = false;
		resetDuixStatus();
	};

	const resetDuixStatus = () => {
		humanIsLoading.value = true;
		isReceiveRes.value = false;
		// isSpeaking.value = false;
		digitalHumanIsShow.value = false;
		duix?.stop();
	};
	const getPlainText = (item) => {
		let result = '';
		const knowledgeText = item.knowledge.reduce((acc, cur) => {
			const mdText = cur.answer;
			const linkText = cur.metadata?.Title;
			if (linkText) {
				return `${mdText}\n\n${linkText}`;
			}
			return acc + mdText;
		}, '');
		// const conclusionText =
		// 	item.conclusion
		// 		?.filter((item) => !!item.report)
		// 		.map((item) => item.report)
		// 		.join('\n\n') ?? '';
		// result += knowledgeText + conclusionText;
		result = knowledgeText;
		return markdownToTxt(result);
	};

	let isWaitingSpeak = false;
	const initDuix = () => {
		const sign = duixConfig.sign; // sign由服务端生成
		const conversationId = duixConfig.conversationId; // duix平台会话id
		if (!sign || !conversationId) {
			return alert('参数不能为空');
		}
		duix.on('error', (data) => {
			console.error(data);
		});
		duix.on('intialSucccess', () => {
			console.info('intialSucccess');
			// 此时初始化成功，可调用start
			duix.start({ conversationId, openAsr: true }).then((res) => {
				console.info('start', res);
			});
		});
		duix.on('bye', (data) => {
			console.info('bye', data);
		});
		duix.on('progress', (progress) => {
			console.info('progress', progress);
		});
		duix.on('show', () => {
			console.info('show');
			humanIsLoading.value = false;
			// 此时可确认视频已
			// (document.querySelector('#modal') as HTMLElement).style.display = 'none';
		});
		duix.on('openAsrSuccess', () => {
			console.info('openAsrSuccess');
		});
		duix.on('asrClose', () => {
			console.info('asrClose');
		});
		duix.on('speakStart', (data) => {
			// isSpeaking.value = true;
			console.info('speakStart', data);
		});
		duix.on('speakEnd', (data) => {
			isReceiveRes.value = false;
		});
		duix.on('speakSection', (data) => {
			console.info('speakSection', data);
		});
		duix.on('speakError', (data) => {
			console.info('speakError', data);
		});
		duix.on('asrResult', (data) => {
			console.info('asrResult', data);

			if (isReceiveRes.value) {
				return;
			}

			let hasResult = false;
			isReceiveRes.value = true;
			try {
				// isWaitingSpeak = true;
				// duix.speak({
				// 	content: '已收到您的问题，正在思考中...请稍等',
				// });
				questionStreamByPost(
					{
						question: data,
						history_group_id: activeRoomId.value,
						raw_mode: false,
						group_type: activeGroupType.value,
						is_digital_human: true,
					},
					(chunkRes) => {
						if (chunkRes.mode === 'result' && chunkRes.value?.answer_type === 'knowledge') {
							const plainText = getPlainText(chunkRes.value);
							hasResult = true;
							duix.speak({
								content: plainText,
							});
						}

						if (!chunkRes.value?.json_ok && chunkRes.value?.err_code === 'MESSAGE') {
							hasResult = true;
							isWaitingSpeak = false;

							duix.speak({
								content: chunkRes.value.json_msg,
							});
						}

						if (chunkRes.mode === 'finish') {
							if (!hasResult) {
								isWaitingSpeak = false;
								duix.speak({
									content: '暂时无法口头描述你所说的问题',
								});
							} else {
								hasResult = false;
							}
							// isReceiveRes.value = false;
						}
					}
				);
			} catch (error) {
				console.error(error);
				isReceiveRes.value = false;
			}
		});
		duix.on('report', (data) => {
			// console.info('report', data)
		});
		duix
			.init({
				sign,
				containerLable: container,
			})
			.then((data) => {
				console.info('init', data);
			});
	};

	let hasInitDuix = false;
	let duix: any;
	const openDigitalHuman = () => {
		digitalHumanIsShow.value = true;

		nextTick(async () => {
			duixConfig.sign = await createSig(duixConfig.appId, duixConfig.appKey, 60 * 60 * duixConfig.expired);
			if (!hasInitDuix) {
				hasInitDuix = true;

				duix = new DUIX();
				initDuix();
			} else {
				duix.start({ conversationId: duixConfig.conversationId, openAsr: true }).then((res) => {
					console.info('start', res);
				});
			}
		});
	};

	function createSig(appId, appKey, sigExp) {
		const now = Math.floor(Date.now() / 1000);
		const expiresAt = now + sigExp;

		const sign = new SignJWT({ appId })
			.setProtectedHeader({ alg: 'HS256' })
			.setIssuedAt(now)
			.setExpirationTime(expiresAt)
			.sign(new TextEncoder().encode(appKey));
		return sign;
	}

	onMounted(() => {
		window.addEventListener('beforeunload', () => {
			closeDigitalHuman();
		});
	});

	onDeactivated(() => {
		closeDigitalHuman();
	});

	return {
		digitalHumanIsShow,
		openDigitalHuman,
		isHumanTalking: isReceiveRes,
		closeDigitalHuman,
		humanIsLoading,
	};
};
